<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/1/24 14:54
 */
namespace Workerman\iDste;
use common\classes\Common;
use common\classes\Error;
use common\classes\HttpRequest;
use common\models\BaseInfo;
use common\models\ProjectorType;
use common\models\SystemLog;
use Workerman\Lib\Timer;
use Workerman\Worker;
use Yii;

require_once __DIR__ . '/../Autoloader.php';

class SystemDeamon
{
    private $projectorTimer;
    public function __construct(){
        $udpWorker = new Worker('udp://127.0.0.1:' . Common::SYSTEM_DEAMON_PORT);
        $udpWorker->onMessage = [$this, 'onUdpMessage'];
        $udpWorker->listen();
        $this->projectorTimer = Timer::add(60, [$this, 'projectorUpdate']);
    }

    public function onUdpMessage($conn, $data){
        echo "network receive message:$data\n";
        $para = json_decode($data, true);
        $ret = '';
        if(!empty($para['action']) && !empty($para['controller'])) {
            switch ($para['controller']) {
                case 'system':
                    $ret = $this->systemCtrl($para['action'], $para['param']);
                    break;
                default:
                    $ret = Error::serializeResponse(Error::STATUS_FAILED);
            }
        }
        else
            $ret = '';
        echo "network deamon return:$ret\n";
        $conn->send($ret);
        $conn->close();
    }


    public function projectorUpdate(){
        $server = 'https://' . Yii::$app->base->cloudServer;
        $req = new HttpRequest($server . '/api/projector/list', [
            'access_token' => Yii::$app->base->accessToken,
            'uuid' => Yii::$app->base->uuid
        ]);
        $ret = $req->get(true);
        if(is_null($ret) || !isset($ret['status']) || $ret['status'] !== Error::STATUS_SUCCESS || !isset($ret['data']))
            return ;
        foreach ($ret['data'] as $item) {
            if(empty($item['cmd_close']) || !ctype_xdigit($item['cmd_close']) ||
                empty($item['cmd_open']) || !ctype_xdigit($item['cmd_open']) ||
                (!empty($item['cmd_pc']) && !ctype_xdigit($item['cmd_pc'])) ||
                (!empty($item['cmd_video']) && !ctype_xdigit($item['cmd_video'])))
                continue;
            $projector = ProjectorType::findOne(['Type' => $item['type']]);
            if(empty($projector))
                $projector = new ProjectorType();
            else{
                continue;
            }
            $projector->Type = $item['type'];
            $projector->PoweroffDelay = $item['delay'];
            $projector->CmdClose = $item['cmd_close'];
            $projector->CmdOpen = $item['cmd_open'];
            $projector->CmdFromPc = $item['cmd_pc'];
            $projector->CmdFromVideo = $item['cmd_video'];
            $projector->StopBit = array_search($item['stop_bit'], ProjectorType::STOP_BIT);
            $projector->DataBit = array_search($item['data_bit'], ProjectorType::DATA_BIT);
            $projector->CheckBit = array_search($item['check_bit'], ProjectorType::CHECK_BIT);
            $projector->BaudRate = array_search($item['baudrate'], ProjectorType::BAUD_RATE);
            $projector->save();
        }
        Timer::del($this->projectorTimer);
        $this->projectorTimer = Timer::add(3600, [$this, 'projectorUpdate']);
    }

    public function restartNginx(){
        echo "restart nginx\n";
        shell_exec(`service nginx restart`);
    }
    private function systemCtrl($action, $param){
        switch ($action){
            case "s-c-u":
                $f = '/etc/nginx/ssl/idste.';
                if(empty($param['crt']) || empty($param['key']))
                    break;
                if(file_put_contents($f . "crt1", $param['crt']) && file_put_contents($f . "key1", $param['key'])){
                    shell_exec("mv $f"."crt1 $f" . "crt");
                    shell_exec("mv $f"."key1 $f" . "key");
                }
                $f = '/usr/local/share/libwebsockets-test-server/libwebsockets-test-server.';
                if(file_put_contents($f . "pem1", $param['crt']) && file_put_contents($f . "key.pem1", $param['key'])){
                    shell_exec("mv $f"."pem1 $f" . "pem");
                    shell_exec("mv $f"."key.pem1 $f" . "key.pem");
                }
                Timer::add(5, [$this, "restartNginx"], null, false);
                return Error::serializeResponse(Error::STATUS_SUCCESS);
        }
        return Error::serializeResponse(Error::STATUS_PARAM_ERROR);
    }

}
